#include <stdio.h>
#include <stdlib.h>
#include "appinst/appinst.h"

AppInst_ObservableObject* initialize_instrumentation() {
	AppInst_InstrumentationService *service = NULL;
	AppInst_ObservableSchema *schema = NULL;
	AppInst_ObservableScope *scope = NULL;
	AppInst_ObservableObject * obj = NULL;
	AppInst_ReturnCode_t retcode = APPINST_RETCODE_ERROR;

	service = AppInst_Infrastructure_create_service(
				"TestService",
				&APPINST_INSTRUMENTATION_SERVICE_PROPERTIES_DEFAULT,
				&retcode);
	if (retcode != APPINST_RETCODE_OK) {
		goto error;
	}

	schema = AppInst_InstrumentationService_create_observable_schema(
				service,
				"TestSchema",
				&APPINST_OBSERVABLE_SCHEMA_PROPERTIES_DEFAULT,
				&retcode);
	if (retcode != APPINST_RETCODE_OK) {
		goto error;
	}

	AppInst_ObservableSchema_create_field(
				schema,
				"i",
				&APPINST_FIELD_PROPERTIES_DEFAULT,
				&retcode);
	if (retcode != APPINST_RETCODE_OK) {
		goto error;
	}

	AppInst_ObservableSchema_create_field(
				schema,
				"instrumented_var",
				&APPINST_FIELD_PROPERTIES_DEFAULT,
				&retcode);
	if (retcode != APPINST_RETCODE_OK) {
		goto error;
	}

	scope = AppInst_InstrumentationService_create_observable_scope(
				service,
				"TestScope",
				&APPINST_OBSERVABLE_SCOPE_PROPERTIES_DEFAULT,
				&retcode);
	if (retcode != APPINST_RETCODE_OK) {
		goto error;
	}

	obj = AppInst_ObservableScope_create_observable_object(
				scope,
				"TestObject",
				&APPINST_OBSERVABLE_OBJECT_PROPERTIES_DEFAULT,
				&retcode);
	if (retcode != APPINST_RETCODE_OK) {
		goto error;
	}

	retcode = AppInst_ObservableScope_enable_data_collection(scope);
	if (retcode != APPINST_RETCODE_OK) {
		goto error;
	}

	return obj;

	error:
	AppInst_Infrastructure_finalize_instrumentation();
	return NULL;
}

int main(
		int argc, char** argv) {
	int i = 0;
	int instrumented_var = 0;

	AppInst_ObservableObject *obj = NULL;

	obj = initialize_instrumentation();

	for (; i > 0; i--) {
		instrumented_var = rand();

		if (obj != NULL) {
			AppInst_ObservableObject_set_value_INT32(obj, 0, i);
			AppInst_ObservableObject_set_value_INT32(obj, 1, instrumented_var);
			AppInst_ObservableObject_save_observation(obj);
		}
	}

	/* finalize instrumentation */
	AppInst_Infrastructure_finalize_instrumentation();

	return 0;
}
