#ifndef APPINST_DATA_REPRESENTATION_H_
#define APPINST_DATA_REPRESENTATION_H_

#include "appinst/appinst_defs.h"

/* ------------------------------------------
 ObservableSchema
 ------------------------------------------ */
const AppInst_STRING8_t
AppInst_ObservableSchema_get_name(
		AppInst_ObservableSchema *self);

const AppInst_Field**
AppInst_ObservableSchema_get_fields(
		AppInst_ObservableSchema *self,
		AppInst_INT32_t* result_len_out);

AppInst_Field*
AppInst_ObservableSchema_lookup_field(
		AppInst_ObservableSchema *self,
		const AppInst_STRING8_t name,
		AppInst_ReturnCode_t *retcode_out);

AppInst_Field*
AppInst_ObservableSchema_create_field(
		AppInst_ObservableSchema *self,
		const AppInst_STRING8_t name,
		const AppInst_FieldProperties_t *properties,
		AppInst_ReturnCode_t *retcode_out);

/* ------------------------------------------
 Field
 ------------------------------------------ */
const AppInst_STRING8_t
AppInst_Field_get_name(
		AppInst_Field *self);

AppInst_INT32_t
AppInst_Field_get_index(
		AppInst_Field *self);

AppInst_DataValueKind_t
AppInst_Field_get_type(
		AppInst_Field *self);

AppInst_INT32_t
AppInst_Field_get_field_max_length(
		AppInst_Field *self);

AppInst_INT32_t
AppInst_Field_get_string_max_length(
		AppInst_Field *self);

/* ------------------------------------------
 Observation
 ------------------------------------------ */
AppInst_ObservableSchema*
AppInst_Observation_get_observable_schema(
		AppInst_Observation *self);

AppInst_INT64_t
AppInst_Observation_get_sequence_number(
		AppInst_Observation *self);

AppInst_UTCTime_t
AppInst_Observation_get_generation_timestamp(
		AppInst_Observation *self);

AppInst_ReturnCode_t
AppInst_Observation_set_flag(
		AppInst_Observation *self,
		AppInst_ObservationFlagKind_t flag);

AppInst_ReturnCode_t
AppInst_Observation_unset_flag(
		AppInst_Observation *self,
		AppInst_ObservationFlagKind_t flag);

AppInst_BOOL_t
AppInst_Observation_is_flag_set(
		AppInst_Observation *self,
		AppInst_ObservationFlagKind_t flag);

AppInst_ReturnCode_t
AppInst_Observation_reset_flags(
		AppInst_Observation *self);

AppInst_BOOL_t
AppInst_Observation_get_value_BOOL(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_ReturnCode_t *retcode_out);

AppInst_OCTET_t
AppInst_Observation_get_value_OCTET(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_ReturnCode_t *retcode_out);

AppInst_INT16_t
AppInst_Observation_get_value_INT16(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_ReturnCode_t *retcode_out);

AppInst_INT32_t
AppInst_Observation_get_value_INT32(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_ReturnCode_t *retcode_out);

AppInst_INT64_t
AppInst_Observation_get_value_INT64(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_ReturnCode_t *retcode_out);

AppInst_UINT16_t
AppInst_Observation_get_value_UINT16(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_ReturnCode_t *retcode_out);

AppInst_UINT32_t
AppInst_Observation_get_value_UINT32(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_ReturnCode_t *retcode_out);

AppInst_UINT64_t
AppInst_Observation_get_value_UINT64(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_ReturnCode_t *retcode_out);

AppInst_FLOAT32_t
AppInst_Observation_get_value_FLOAT32(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_ReturnCode_t *retcode_out);

AppInst_FLOAT64_t
AppInst_Observation_get_value_FLOAT64(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_ReturnCode_t *retcode_out);

AppInst_FLOAT128_t
AppInst_Observation_get_value_FLOAT128(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_ReturnCode_t *retcode_out);

AppInst_CHAR8_t
AppInst_Observation_get_value_CHAR8(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_ReturnCode_t *retcode_out);

AppInst_STRING8_t
AppInst_Observation_get_value_STRING8(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_ReturnCode_t *retcode_out);

#ifdef APPINST_ENABLE_WCHAR
AppInst_CHAR32_t
AppInst_Observation_get_value_CHAR32(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_ReturnCode_t *retcode_out);

AppInst_STRING32_t
AppInst_Observation_get_value_STRING32(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_ReturnCode_t *retcode_out);
#endif

AppInst_BOOLSeq_t
AppInst_Observation_get_value_BOOLSeq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t* result_len_out,
		AppInst_ReturnCode_t *retcode_out);

AppInst_OCTETSeq_t
AppInst_Observation_get_value_OCTETSeq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t* result_len_out,
		AppInst_ReturnCode_t *retcode_out);

AppInst_INT16Seq_t
AppInst_Observation_get_value_INT16Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t* result_len_out,
		AppInst_ReturnCode_t *retcode_out);

AppInst_INT32Seq_t
AppInst_Observation_get_value_INT32Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t* result_len_out,
		AppInst_ReturnCode_t *retcode_out);

AppInst_INT64Seq_t
AppInst_Observation_get_value_INT64Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t* result_len_out,
		AppInst_ReturnCode_t *retcode_out);

AppInst_UINT16Seq_t
AppInst_Observation_get_value_UINT16Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t* result_len_out,
		AppInst_ReturnCode_t *retcode_out);

AppInst_UINT32Seq_t
AppInst_Observation_get_value_UINT32Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t* result_len_out,
		AppInst_ReturnCode_t *retcode_out);

AppInst_UINT64Seq_t
AppInst_Observation_get_value_UINT64Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t* result_len_out,
		AppInst_ReturnCode_t *retcode_out);

AppInst_FLOAT32Seq_t
AppInst_Observation_get_value_FLOAT32Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t* result_len_out,
		AppInst_ReturnCode_t *retcode_out);

AppInst_FLOAT64Seq_t
AppInst_Observation_get_value_FLOAT64Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t* result_len_out,
		AppInst_ReturnCode_t *retcode_out);

AppInst_FLOAT128Seq_t
AppInst_Observation_get_value_FLOAT128Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t* result_len_out,
		AppInst_ReturnCode_t *retcode_out);

AppInst_CHAR8Seq_t
AppInst_Observation_get_value_CHAR8Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t* result_len_out,
		AppInst_ReturnCode_t *retcode_out);

AppInst_STRING8Seq_t
AppInst_Observation_get_value_STRING8Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t* result_len_out,
		AppInst_ReturnCode_t *retcode_out);

#ifdef APPINST_ENABLE_WCHAR
AppInst_CHAR32Seq_t
AppInst_Observation_get_value_CHAR32Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t* result_len_out,
		AppInst_ReturnCode_t *retcode_out);

AppInst_STRING32Seq_t
AppInst_Observation_get_value_STRING32Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t* result_len_out,
		AppInst_ReturnCode_t *retcode_out);
#endif

AppInst_ReturnCode_t
AppInst_Observation_set_value_BOOL(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_BOOL_t value);

AppInst_ReturnCode_t
AppInst_Observation_set_value_OCTET(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_OCTET_t value);

AppInst_ReturnCode_t
AppInst_Observation_set_value_INT16(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT16_t value);

AppInst_ReturnCode_t
AppInst_Observation_set_value_INT32(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32_t value);

AppInst_ReturnCode_t
AppInst_Observation_set_value_INT64(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT64_t value);

AppInst_ReturnCode_t
AppInst_Observation_set_value_UINT16(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_UINT16_t value);

AppInst_ReturnCode_t
AppInst_Observation_set_value_UINT32(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_UINT32_t value);

AppInst_ReturnCode_t
AppInst_Observation_set_value_UINT64(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_UINT64_t value);

AppInst_ReturnCode_t
AppInst_Observation_set_value_FLOAT32(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_FLOAT32_t value);

AppInst_ReturnCode_t
AppInst_Observation_set_value_FLOAT64(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_FLOAT64_t value);

AppInst_ReturnCode_t
AppInst_Observation_set_value_FLOAT128(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_FLOAT128_t value);

AppInst_ReturnCode_t
AppInst_Observation_set_value_CHAR8(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_CHAR8_t value);

AppInst_ReturnCode_t
AppInst_Observation_set_value_STRING8(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_STRING8_t value);

#ifdef APPINST_ENABLE_WCHAR
AppInst_ReturnCode_t
AppInst_Observation_set_value_CHAR32(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_CHAR32_t value);

AppInst_ReturnCode_t
AppInst_Observation_set_value_STRING32(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_STRING32_t value);
#endif

AppInst_ReturnCode_t
AppInst_Observation_set_value_BOOLSeq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_BOOLSeq_t value,
		AppInst_INT32_t value_len);

AppInst_ReturnCode_t
AppInst_Observation_set_value_OCTETSeq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_OCTETSeq_t value,
		AppInst_INT32_t value_len);

AppInst_ReturnCode_t
AppInst_Observation_set_value_INT16Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT16Seq_t value,
		AppInst_INT32_t value_len);

AppInst_ReturnCode_t
AppInst_Observation_set_value_INT32Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT32Seq_t value,
		AppInst_INT32_t value_len);

AppInst_ReturnCode_t
AppInst_Observation_set_value_INT64Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_INT64Seq_t value,
		AppInst_INT32_t value_len);

AppInst_ReturnCode_t
AppInst_Observation_set_value_UINT16Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_UINT16Seq_t value,
		AppInst_INT32_t value_len);

AppInst_ReturnCode_t
AppInst_Observation_set_value_UINT32Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_UINT32Seq_t value,
		AppInst_INT32_t value_len);

AppInst_ReturnCode_t
AppInst_Observation_set_value_UINT64Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_UINT64Seq_t value,
		AppInst_INT32_t value_len);

AppInst_ReturnCode_t
AppInst_Observation_set_value_FLOAT32Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_FLOAT32Seq_t value,
		AppInst_INT32_t value_len);

AppInst_ReturnCode_t
AppInst_Observation_set_value_FLOAT64Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_FLOAT64Seq_t value,
		AppInst_INT32_t value_len);

AppInst_ReturnCode_t
AppInst_Observation_set_value_FLOAT128Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_FLOAT128Seq_t value,
		AppInst_INT32_t value_len);

AppInst_ReturnCode_t
AppInst_Observation_set_value_CHAR8Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_CHAR8Seq_t value,
		AppInst_INT32_t value_len);

AppInst_ReturnCode_t
AppInst_Observation_set_value_STRING8Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_STRING8Seq_t value,
		AppInst_INT32_t value_len);

#ifdef APPINST_ENABLE_WCHAR
AppInst_ReturnCode_t
AppInst_Observation_set_value_CHAR32Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_CHAR32Seq_t value,
		AppInst_INT32_t value_len);

AppInst_ReturnCode_t
AppInst_Observation_set_value_STRING32Seq(
		AppInst_Observation *self,
		AppInst_INT32_t field_id,
		AppInst_STRING32Seq_t value,
		AppInst_INT32_t value_len);
#endif

/* ------------------------------------------
 ObservationFlagKind
 ------------------------------------------ */
enum AppInst_ObservationFlagKind {
	APPINST_OBSV_FLAG_LOCAL,
	APPINST_OBSV_FLAG_KEEP
};

#endif /* APPINST_DATA_REPRESENTATION_H_ */
