#ifndef APPINST_TYPES_H_
#define APPINST_TYPES_H_

#ifdef APPINST_ENABLE_WCHAR
#include <wchar.h>
#endif


/* ------------------------------------------
 Data Value Module
 ------------------------------------------ */

typedef
	enum AppInst_DataValueKind
	AppInst_DataValueKind_t;

typedef
	unsigned char
	AppInst_BOOL_t;

typedef
	unsigned char
	AppInst_OCTET_t;

typedef
	short
	AppInst_INT16_t;

typedef
	long
	AppInst_INT32_t;

typedef
	long long
	AppInst_INT64_t;

typedef
	unsigned short
	AppInst_UINT16_t;

typedef
	unsigned long
	AppInst_UINT32_t;

typedef
	unsigned long long
	AppInst_UINT64_t;

typedef
	float
	AppInst_FLOAT32_t;

typedef
	double
	AppInst_FLOAT64_t;

typedef
	long double
	AppInst_FLOAT128_t;

typedef
	char
	AppInst_CHAR8_t;

typedef
	char*
	AppInst_STRING8_t;


#ifdef APPINST_ENABLE_WCHAR
typedef
	wchar_t
	AppInst_CHAR32_t;

typedef
	wchar_t*
	AppInst_STRING32_t;
#endif

typedef
	AppInst_BOOL_t*
	AppInst_BOOLSeq_t;

typedef
	AppInst_OCTET_t*
	AppInst_OCTETSeq_t;

typedef
	AppInst_INT16_t*
	AppInst_INT16Seq_t;

typedef
	AppInst_INT32_t*
	AppInst_INT32Seq_t;

typedef
	AppInst_INT64_t*
	AppInst_INT64Seq_t;

typedef
	AppInst_UINT16_t*
	AppInst_UINT16Seq_t;

typedef
	AppInst_UINT32_t*
	AppInst_UINT32Seq_t;

typedef
	AppInst_UINT64_t*
	AppInst_UINT64Seq_t;

typedef
	AppInst_FLOAT32_t*
	AppInst_FLOAT32Seq_t;

typedef
	AppInst_FLOAT64_t*
	AppInst_FLOAT64Seq_t;

typedef
	AppInst_FLOAT128_t*
	AppInst_FLOAT128Seq_t;

typedef
	AppInst_CHAR8_t*
	AppInst_CHAR8Seq_t;

typedef
	AppInst_STRING8_t*
	AppInst_STRING8Seq_t;

#ifdef APPINST_ENABLE_WCHAR
typedef
	AppInst_CHAR32_t*
	AppInst_CHAR32Seq_t;

typedef
	AppInst_STRING32_t*
	AppInst_STRING32Seq_t;
#endif

typedef
	void*
	AppInst_DataValueSource;

typedef
	enum AppInst_ReturnCode
	AppInst_ReturnCode_t;

typedef
	struct AppInst_Time
	AppInst_Time_t;

typedef
	struct AppInst_UTCTime
	AppInst_UTCTime_t;

/* ------------------------------------------
 Properties Module
 ------------------------------------------ */

typedef
	struct AppInst_DefaultConfigurationTableI
	AppInst_DefaultConfigurationTable;

typedef
	struct AppInst_InstrumentationServiceProperties
	AppInst_InstrumentationServiceProperties_t;

typedef
	struct AppInst_ObservableSchemaProperties
	AppInst_ObservableSchemaProperties_t;

typedef
	struct AppInst_FieldProperties
	AppInst_FieldProperties_t;

typedef
	struct AppInst_ObservableObjectProperties
	AppInst_ObservableObjectProperties_t;

typedef
	struct AppInst_ObservableScopeProperties
	AppInst_ObservableScopeProperties_t;

typedef
	struct AppInst_DataProcessorProperties
	AppInst_DataProcessorProperties_t;



/* ------------------------------------------
 Instrumentation Module
 ------------------------------------------ */

typedef
	struct AppInst_InstrumentationServiceI
	AppInst_InstrumentationService;

/* ------------------------------------------
 Data Representation Module
 ------------------------------------------ */

typedef
	struct AppInst_ObservableSchemaI
	AppInst_ObservableSchema;

typedef
	struct AppInst_FieldI
	AppInst_Field;

typedef
	struct AppInst_ObservationI
	AppInst_Observation;

typedef
	enum AppInst_ObservationFlagKind
	AppInst_ObservationFlagKind_t;


/* ------------------------------------------
 Data Collection Module
 ------------------------------------------ */

typedef
	struct AppInst_ObservableObjectI
	AppInst_ObservableObject;

typedef
	struct AppInst_ObservableScopeI
	AppInst_ObservableScope;

typedef
	struct AppInst_DataProcessorI
	AppInst_DataProcessor;

typedef
	struct AppInst_DataProcessorArgs
	AppInst_DataProcessorArgs_t;

typedef
	void*
	AppInst_DataProcessorState;

typedef
	AppInst_ReturnCode_t (*AppInst_DataProcessor_initialize_fn)(
		AppInst_DataProcessor *self,
		AppInst_InstrumentationService *service,
		AppInst_DataProcessorArgs_t *init_args);

typedef
	AppInst_ReturnCode_t
	(*AppInst_DataProcessor_finalize_fn)(
		AppInst_DataProcessor *self,
		AppInst_InstrumentationService *service);

typedef
	AppInst_ReturnCode_t
	(*AppInst_DataProcessor_update_fn)(
		AppInst_DataProcessor *self,
		AppInst_InstrumentationService *service,
		AppInst_DataProcessorArgs_t *args);

typedef
	AppInst_DataProcessorState
	(*AppInst_DataProcessor_attach_to_observable_scope_fn)(
		AppInst_DataProcessor *self,
		AppInst_ObservableScope *scope,
		AppInst_ReturnCode_t *retcode_out);

typedef
	AppInst_ReturnCode_t
	(*AppInst_DataProcessor_detach_from_observable_scope_fn)(
		AppInst_DataProcessor *self,
		AppInst_ObservableScope *scope,
		AppInst_DataProcessorState scope_state);

typedef
	AppInst_DataProcessorState
	(*AppInst_DataProcessor_attach_to_observable_object_fn)(
		AppInst_DataProcessor *self,
		AppInst_ObservableObject *obj,
		AppInst_DataProcessorState scope_state,
		AppInst_ReturnCode_t *retcode_out);

typedef
	AppInst_ReturnCode_t
	(*AppInst_DataProcessor_detach_from_observable_object_fn)(
		AppInst_DataProcessor *self,
		AppInst_ObservableObject *obj,
		AppInst_DataProcessorState scope_state,
		AppInst_DataProcessorState obj_state);

typedef
	AppInst_ReturnCode_t
	(*AppInst_DataProcessor_process_observations_fn)(
		AppInst_DataProcessor *self,
		const AppInst_Observation **observations,
		AppInst_ObservableObject *obj,
		AppInst_DataProcessorState scope_state,
		AppInst_DataProcessorState object_state,
		int observations_len);


#endif /* APPINST_TYPES_H_ */
