#ifndef APPINST_PROPERTIES_H_
#define APPINST_PROPERTIES_H_

#include "appinst/appinst_defs.h"

/* ------------------------------------------
 DefaultConfigurationTable
 ------------------------------------------ */

AppInst_DefaultConfigurationTable*
AppInst_DefaultConfigurationTable_get_instance();

const AppInst_InstrumentationServiceProperties_t*
AppInst_DefaultConfigurationTable_get_default_service_properties(
		AppInst_DefaultConfigurationTable *self);

AppInst_ReturnCode_t
AppInst_DefaultConfigurationTable_set_default_service_properties(
		AppInst_DefaultConfigurationTable *self,
		const AppInst_InstrumentationServiceProperties_t *properties);

const AppInst_ObservableScopeProperties_t*
AppInst_DefaultConfigurationTable_get_default_observable_scope_properties(
		AppInst_DefaultConfigurationTable *self);

AppInst_ReturnCode_t
AppInst_DefaultConfigurationTable_set_default_observable_scope_properties(
		AppInst_DefaultConfigurationTable *self,
		const AppInst_ObservableScopeProperties_t *properties);

const AppInst_ObservableObjectProperties_t*
AppInst_DefaultConfigurationTable_get_default_observable_object_properties(
		AppInst_DefaultConfigurationTable *self);

AppInst_ReturnCode_t
AppInst_DefaultConfigurationTable_set_default_observable_object_properties(
		AppInst_DefaultConfigurationTable *self,
		const AppInst_ObservableObjectProperties_t *properties);

const AppInst_ObservableSchemaProperties_t*
AppInst_DefaultConfigurationTable_get_default_observable_schema_properties(
		AppInst_DefaultConfigurationTable *self);

AppInst_ReturnCode_t
AppInst_DefaultConfigurationTable_set_default_observable_schema_properties(
		AppInst_DefaultConfigurationTable *self,
		const AppInst_ObservableSchemaProperties_t *properties);

const AppInst_DataProcessorProperties_t*
AppInst_DefaultConfigurationTable_get_default_data_processor_properties(
		AppInst_DefaultConfigurationTable *self);

AppInst_ReturnCode_t
AppInst_DefaultConfigurationTable_set_default_data_processor_properties(
		AppInst_DefaultConfigurationTable *self,
		const AppInst_DataProcessorProperties_t *properties);

/* ------------------------------------------
 InstrumentationServiceProperties
 ------------------------------------------ */

struct AppInst_InstrumentationServiceProperties { };
#define APPINST_INSTRUMENTATION_SERVICE_PROPERTIES_INITIALIZER { }

const AppInst_InstrumentationServiceProperties_t
	APPINST_INSTRUMENTATION_SERVICE_PROPERTIES_DEFAULT =
			APPINST_INSTRUMENTATION_SERVICE_PROPERTIES_INITIALIZER;

/* ------------------------------------------
 ObservableSchemaProperties
 ------------------------------------------ */
struct AppInst_ObservableSchemaProperties { };

#define APPINST_OBSERVABLE_SCHEMA_PROPERTIES_INITIALIZER { }

const AppInst_ObservableSchemaProperties_t
	APPINST_OBSERVABLE_SCHEMA_PROPERTIES_DEFAULT =
			APPINST_OBSERVABLE_SCHEMA_PROPERTIES_INITIALIZER;

/* ------------------------------------------
 FieldProperties
 ------------------------------------------ */
struct AppInst_FieldProperties {
	AppInst_DataValueKind_t type;
	AppInst_INT32_t field_max_length;
	AppInst_INT32_t string_max_size;
};

#define APPINST_FIELD_PROPERTIES_INITIALIZER \
{ \
			APPINST_TYPE_INT32,\
			1,\
			1\
}

const AppInst_FieldProperties_t
	APPINST_FIELD_PROPERTIES_DEFAULT =
			APPINST_FIELD_PROPERTIES_INITIALIZER;

/* ------------------------------------------
 ObservableScopeProperties
 ------------------------------------------ */
struct AppInst_ObservableScopeProperties {
	AppInst_BOOL_t enable_data_collection;
};

#define APPINST_OBSERVABLE_SCOPE_PROPERTIES_INITIALIZER \
{\
	APPINST_BOOL_FALSE\
}

const AppInst_ObservableScopeProperties_t
	APPINST_OBSERVABLE_SCOPE_PROPERTIES_DEFAULT =
			APPINST_OBSERVABLE_SCOPE_PROPERTIES_INITIALIZER;

/* ------------------------------------------
 ObservableObjectProperties
 ------------------------------------------ */
struct AppInst_ObservableObjectProperties {
	AppInst_STRING8_t observable_schema_name;
	AppInst_BOOL_t enable_save_observation;
	AppInst_BOOL_t take_observation_timestamp;
	AppInst_Time_t observation_max_blocking_time;
	AppInst_INT32_t allocated_observations_initial;
	AppInst_INT32_t allocated_observations_max;
	AppInst_STRING8_t data_processor_name;
};

#define APPINST_OBSERVABLE_OBJECT_PROPERTIES_INITIALIZER \
{\
	NULL,\
	APPINST_BOOL_TRUE,\
	APPINST_BOOL_TRUE,\
	{ 0, 0 }, \
	0, \
	-1,\
	NULL\
}

const AppInst_ObservableObjectProperties_t
	APPINST_OBSERVABLE_OBJECT_PROPERTIES_DEFAULT =
			APPINST_OBSERVABLE_OBJECT_PROPERTIES_INITIALIZER;

/* ------------------------------------------
 DataProcessorArgs
 ------------------------------------------ */
struct AppInst_DataProcessorArgs {
	AppInst_STRING8Seq_t keys;
	AppInst_STRING8Seq_t values;
	int len;
};


#define APPINST_DATA_PROCESSOR_ARGS_INITIALIZER \
{\
	NULL,\
	NULL,\
	0\
}

/* ------------------------------------------
 DataProcessorProperties
 ------------------------------------------ */
struct AppInst_DataProcessorProperties {
	AppInst_DataProcessorArgs_t init_args;
	AppInst_DataProcessor_initialize_fn initialize_fn;
	AppInst_DataProcessor_finalize_fn finalize_fn;
	AppInst_DataProcessor_update_fn update_fn;
	AppInst_DataProcessor_attach_to_observable_scope_fn attach_to_observable_scope_fn;
	AppInst_DataProcessor_detach_from_observable_scope_fn detach_from_observable_scope_fn;
	AppInst_DataProcessor_attach_to_observable_object_fn attach_to_observable_object_fn;
	AppInst_DataProcessor_detach_from_observable_object_fn detach_from_observable_object_fn;
	AppInst_DataProcessor_process_observations_fn process_observations_fn;
};

#define APPINST_DATA_PROCESSOR_PROPERTIES_INITIALIZER \
{ \
	APPINST_DATA_PROCESSOR_ARGS_INITIALIZER, \
	NULL, \
	NULL, \
	NULL, \
	NULL, \
	NULL, \
	NULL, \
	NULL, \
	NULL\
}

const AppInst_DataProcessorProperties_t
	APPINST_DATA_PROCESSOR_PROPERTIES_DEFAULT =
			APPINST_DATA_PROCESSOR_PROPERTIES_INITIALIZER;


#endif /* APPINST_PROPERTIES_H_ */
